using System;
using System.Collections.Generic;
using System.Text;
using Utils.Logs.Exceptions;
using System.Collections;

namespace Utils.Logs
{
    /// <summary>
    /// A log entry
    /// </summary>
    public class LogRow
    {
        #region Variables
        /// <summary>
        /// Identifacator
        /// </summary>
        private int vID = -1;
        /// <summary>
        /// Date of the log entry
        /// </summary>
        private DateTime vDate;
        /// <summary>
        /// Type of the log entry
        /// </summary>
        private LogType vType;
        /// <summary>
        /// Title of the log entry
        /// </summary>
        private string vTitle;
        /// <summary>
        /// Message of the log entry
        /// </summary>
        private string vMessage;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public LogRow() { }
        /// <summary>
        /// Constructor (try to parse the _LogEntry
        /// </summary>
        /// <param name="_LogEntry">Logs entry that will be parsed</param>
        /// <exception cref="BadLogParseException">Impossible to parse the log</exception>
        public LogRow(string _LogEntry)
        {
            try { Fill(_LogEntry); }
            catch (BadLogParseException) { throw; }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the identificator of the log entry
        /// </summary>
        public int ID { get { return vID; } set { vID = value; } }
        /// <summary>
        /// Gets or sets the date of the log entry
        /// </summary>        
        public DateTime Date
        {
            get { return vDate; }
            set { vDate = value; }
        }
        /// <summary>
        /// Give the date in string mode
        /// </summary>
        public string ParsedDate { get { return vDate.ToString("dd/MM/yyyy [HH:mm:ss]"); } }
        /// <summary>
        /// Gets or sets the message of the log entry
        /// </summary>
        public LogType Type
        {
            get { return vType; }
            set { vType = value; }
        }
        /// <summary>
        /// Gets or sets the title of the log entry
        /// </summary>
        public string Title
        {
            get { return vTitle; }
            set { vTitle = value; }
        }
        /// <summary>
        /// Gets or sets the type of the log entry
        /// </summary>
        public string Message
        {
            get { return vMessage; }
            set { vMessage = value; }
        }
        /// <summary>
        /// Gets an array wich contains all the informations of the LogRow
        /// </summary>
        public string[] Entry
        {
            get
            {
                string Date = this.Date.ToShortDateString() + "[" + this.Date.ToLongTimeString() + "]";
                string[] ret = new string[4];
                ret[0] = Date;
                ret[1] = this.Type.ToString();
                ret[2] = this.Title;
                ret[3] = this.Message;
                return ret;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Try to fill if possible the LogRow with the data of the string _LogEntry
        /// </summary>
        /// <param name="_LogEntry">The log entry to parse</param>
        public bool TryFill(string _LogEntry)
        {
            LogRow lr = new LogRow();
            bool ret = TryParse(ref lr, _LogEntry);
            return ret;
        }
        /// <summary>
        /// Fill if possible the LogRow with the data of the string _LogEntry
        /// </summary>
        /// <exception cref="BadLogParseException">If the parse failed</exception>
        public void Fill(string _LogEntry)
        {
            LogRow le = Parse(_LogEntry);
            this.Type = le.Type;
            this.Title = le.Title;
            this.Message = le.Message;
        }
        /// <summary>
        /// Try to parse the string entry of log and fill if possible the object
        /// </summary>
        /// <param name="_LogRow">LogRow to fill with the string</param>
        /// <param name="_LogEntry">The log entry to parse</param>
        public static bool TryParse(ref LogRow _LogRow, string _LogEntry)
        {
            _LogEntry = _LogEntry.Replace(Log.LineSeparator, "");
            string[] Options = { Log.CellSeparator };
            string[] Cells = _LogEntry.Split(Options, StringSplitOptions.RemoveEmptyEntries);

            if (Cells.Length != 4) return false;
            _LogRow.Date = DateTime.Parse(Cells[0]);

            #region TypeLog
            bool exists = false;
            foreach (LogType item in Enum.GetValues(typeof(LogType)))
            {
                Cells[1] = Cells[1].Trim();
                if (Cells[1] == item.ToString())
                {
                    _LogRow.Type = item;
                    exists = true;
                    break;
                }
            }
            if (!exists) return false;
            #endregion

            _LogRow.Title = Cells[2].TrimEnd(new char[] { ' ' });
            _LogRow.Message = Cells[3].TrimEnd(new char[] { ' ' });
            return true;
        }
        /// <summary>
        /// Fill the string entry of log and fill if possible the object
        /// </summary>
        /// <param name="_LogEntry">string Logs entry</param>
        /// <returns>An instance of LogRow</returns>
        /// <exception cref="BadLogParseException">If the parse failed</exception>
        public static LogRow Parse(string _LogEntry)
        {
            LogRow lr = new LogRow();
            if (!TryParse(ref lr, _LogEntry))
                throw new BadLogParseException("Impossible to parse de LogRow");
            else
                return lr;
        }
        /// <summary>
        /// Gives the object in string mode
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return this.Date.ToString() + Log.CellSeparator + this.Type.ToString() + Log.CellSeparator
                + this.Title + Log.CellSeparator + this.Message + Log.CellSeparator + Log.LineSeparator;
        }
        #endregion

        #region Operators
        /// <summary>
        /// Converts the instance of LogRow into a string
        /// </summary>
        /// <param name="_LogRow"></param>
        /// <returns></returns>
        public static implicit operator string(LogRow _LogRow)
        {
            return _LogRow.ToString();
        }
        #endregion
    }

    #region Comparer
    /// <summary>
    /// Compare LogRow on date
    /// </summary>
    public class LogRowDateComparer : IComparer
    {
        #region IComparer Members
        /// <summary>
        /// Comparer on the date of a MeetingRow
        /// </summary>
        /// <param name="x">First object to compare</param>
        /// <param name="y">Second object to compare</param>
        /// <returns>if x > y returns 1 if x == y returns 0 else returns -1 </returns>
        public int Compare(object x, object y)
        {
            DateTime dx = ((LogRow)x).Date;
            DateTime dy = ((LogRow)y).Date;
            if (dx.Ticks > dy.Ticks)
                return 1;
            else if (dx.Ticks == dy.Ticks)
                return 0;
            else
                return -1;
        }

        #endregion
    }
    /// <summary>
    /// Compare LogRow on identificator
    /// </summary>
    public class LogRowIDComparer : IComparer
    {
        #region IComparer Members
        /// <summary>
        /// Comparer on the date of a MeetingRow
        /// </summary>
        /// <param name="x">First object to compare</param>
        /// <param name="y">Second object to compare</param>
        /// <returns>if x > y returns 1 if x == y returns 0 else returns -1 </returns>
        public int Compare(object x, object y)
        {
            int dx = ((LogRow)x).ID;
            int dy = ((LogRow)y).ID;
            if (dx > dy)
                return 1;
            else if (dx == dy)
                return 0;
            else
                return -1;
        }

        #endregion
    }
    #endregion
}
